@@ -1,7 +1,7 @@
 package Perinci::Access::Simple::Server::Pipe;
 
-our $DATE = '2014-12-11'; # DATE
-our $VERSION = '0.20'; # VERSION
+our $DATE = '2015-03-05'; # DATE
+our $VERSION = '0.21'; # VERSION
 
 use 5.010001;
 use strict;
@@ -48,8 +48,7 @@ sub send_response {
     my $v = $self->req->{v} // 1.1;
     insert_riap_stuffs_to_res($res, $v);
     $res = $cleanser->clone_and_clean($res);
-    my $res_json = $json->encode($res);
-    print "J", length($res_json), "\015\012", $res_json, "\015\012";
+    print "j", $json->encode($res), "\015\012";
 }
 
 sub run {
@@ -66,11 +65,8 @@ sub run {
         my $req_json;
         if ($line =~ /\Aj(.*)/) {
             $req_json = $1;
-        } elsif ($line =~ /\AJ(\d+)/) {
-            read STDIN, $req_json, $1;
-            my $crlf = <STDIN>;
         } else {
-            $self->res([400, "Invalid request line, use J<num> or j<json>"]);
+            $self->res([400, "Invalid request line, use j<json>"]);
             $last++;
             goto RES;
         }
@@ -117,7 +113,7 @@ Perinci::Access::Simple::Server::Pipe - (Base) class for creating Riap::Simple s
 
 =head1 VERSION
 
-This document describes version 0.20 of Perinci::Access::Simple::Server::Pipe (from Perl distribution Perinci-Access-Simple-Server), released on 2014-12-11.
+This document describes version 0.21 of Perinci::Access::Simple::Server::Pipe (from Perl distribution Perinci-Access-Simple-Server), released on 2015-03-05.
 
 =head1 SYNOPSIS
 
@@ -204,7 +200,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
