@@ -1,12 +1,13 @@
 package Perinci::Access::Simple::Server::Socket;
 
+our $DATE = '2015-03-05'; # DATE
+our $VERSION = '0.21'; # VERSION
+
 use 5.010001;
 use strict;
 use warnings;
 use Log::Any '$log';
 
-our $VERSION = '0.20'; # VERSION
-
 use Data::Clean::FromJSON;
 use Data::Clean::JSON;
 use File::HomeDir;
@@ -19,7 +20,7 @@ use JSON;
 use Perinci::Access;
 use Perinci::AccessUtil qw(insert_riap_stuffs_to_res decode_args_in_riap_req);
 use Proc::Daemon::Prefork;
-use Time::HiRes qw(gettimeofday tv_interval);
+use Time::HiRes qw(time);
 use URI::Escape;
 
 use Moo;
@@ -221,6 +222,7 @@ sub _main_loop {
 
     my $sel = IO::Select->new(@{ $self->_server_socks });
 
+  CONN:
     for (my $i=1; $i<=$self->max_requests_per_child; $i++) {
         $self->_daemon->set_label("listening");
         my @ready = $sel->can_read();
@@ -229,14 +231,14 @@ sub _main_loop {
             my $sock = $s->accept();
             # sock can be undef
             next unless $sock;
-            $self->{_connect_time} = [gettimeofday];
+            $self->{_connect_time}   = time();
             $self->_set_label_serving($sock);
 
             my $timeout = ${*$sock}{'io_socket_timeout'};
             my $fdset = "";
             vec($fdset, $sock->fileno, 1) = 1;
-            my $buf = "";
 
+            my $last_child = 0;
           REQ:
             while (1) {
                 $self->_daemon->update_scoreboard({
@@ -244,37 +246,21 @@ sub _main_loop {
                     num_reqs => $i,
                     state => "R",
                 });
+                $self->{_start_req_time} = time();
+                my $buf = $self->_sysreadline($sock, $timeout, $fdset);
+                say "D1";
+                last CONN unless defined $buf;
 
-                $self->{_start_req_time} = [gettimeofday];
-              READ_REQ:
-                while (1) {
-                    # loop until we have the whole request in $buf
-                    $buf =~ s/\A(?:\015?\012)+//;  # ignore leading blank lines
-                    #$log->tracef("buf=%s (%d bytes)", $buf, length($buf));
-                    if ($buf =~ /\Aj(.*)\015?\012/) {
-                        $self->{_req_json} = $1;
-                        $log->tracef("Received JSON from client: %s",
-                                     $self->{_req_json});
-                        $buf = substr($buf, 1+length($1));
-                        last READ_REQ;
-                    } elsif ($buf =~ /\AJ(\d+)(\015?\012)(.+)\015?\012/s
-                                 && length($3) >= $1) {
-                        $self->{_req_json} = substr($3, 0, $1);
-                        $log->tracef("Received JSON from client: %s",
-                                     $self->{_req_json});
-                        $buf = substr($buf, 1+length($1)+length($2)+length($3));
-                        last READ_REQ;
-                    } elsif ($buf =~ /\A([^jJ].*)/) {
-                        $log->tracef("Received from client: %s", $1);
-                        $self->{_finish_req_time} = [gettimeofday];
-                        $self->{_res} = [400, "Invalid request line"];
-                        $buf =~ s/\A.+//;
-                        goto FINISH_REQ;
-                    }
-                    last REQ unless $self->_need_more(
-                        $sock, $buf, $timeout, $fdset);
+                $self->{_finish_req_time} = time();
+                $log->tracef("Received line from client: %s", $buf);
+
+                if ($buf =~ /\Aj(.*)\015?\012/) {
+                    $self->{_req_json} = $1;
+                } else {
+                    $self->{_res} = [400, "Invalid request line"];
+                    $last_child++;
+                    goto FINISH_REQ;
                 }
-                $self->{_finish_req_time} = [gettimeofday];
 
                 eval {
                     $self->{_req} = $json->decode($self->{_req_json});
@@ -292,11 +278,11 @@ sub _main_loop {
                 }
 
               RES:
-                $self->{_start_res_time}  = [gettimeofday];
+                $self->{_start_res_time}  = time();
                 $self->{_res} = $self->riap_client->request(
                     $self->{_req}{action} => $self->{_req}{uri},
                     $self->{_req});
-                $self->{_finish_res_time} = [gettimeofday];
+                $self->{_finish_res_time} = time();
 
               FINISH_REQ:
                 $self->_daemon->update_scoreboard({state => "W"});
@@ -308,35 +294,33 @@ sub _main_loop {
                     $self->{_res} = [500, "Can't encode result in JSON: $e"];
                     $self->{_res_json} = $json->encode($self->{_res});
                 }
-                $self->_write_sock($sock, "J" . length($self->{_res_json}).
-                                       "\015\012");
-                $self->_write_sock($sock, $self->{_res_json});
-                $self->_write_sock($sock, "\015\012");
+                $self->_write_sock($sock, "j".$self->{_res_json}."\015\012");
                 $self->access_log($sock);
                 $self->_daemon->update_scoreboard({state => "_"});
-            }
+
+                last CONN if $last_child;
+            } # while REQ
             $sock->close;
-        }
-    }
+        } # for SOCK
+    } # for CONN
 }
 
-sub _need_more {
-    my $self = shift;
-    my $sock = shift;
-    #my($buf,$timeout,$fdset) = @_;
-    if ($_[1]) {
-	my($timeout, $fdset) = @_[1,2];
+sub _sysreadline {
+    my ($self, $sock, $timeout, $fdset) = @_;
+    if ($timeout) {
 	#print STDERR "select(,,,$timeout)\n" if $DEBUG;
-	my $n = select($fdset,undef,undef,$timeout);
+	my $n = select($fdset, undef, undef, $timeout);
 	unless ($n) {
 	    #$self->reason(defined($n) ? "Timeout" : "select: $!");
-	    return;
+	    return undef;
 	}
     }
     #print STDERR "sysread()\n" if $DEBUG;
-    my $n = sysread($sock, $_[0], 2048, length($_[0]));
-    #$self->reason(defined($n) ? "Client closed" : "sysread: $!") unless $n;
-    $n;
+    my $buf = "";
+    while (1) {
+        my $n = sysread($sock, $buf, 2048, length($buf));
+        return $buf if $buf =~ /\012/ || !$n;
+    }
 }
 
 sub _write_sock {
@@ -433,9 +417,8 @@ sub access_log {
     }
 
     my $reqt = sprintf("%.3fms",
-                       1000*tv_interval(
-                           $self->{_start_req_time},
-                           $self->{_finish_req_time}));
+                       1000*($self->{_finish_req_time}-
+                                 $self->{_start_req_time}));
 
     if (!$self->{_res}) {
         warn "BUG: No response generated";
@@ -450,8 +433,8 @@ sub access_log {
     my $rest;
     if ($self->{_finish_res_time}) {
         $rest = sprintf("%.3fms",
-                        1000*tv_interval($self->{_start_res_time},
-                                         $self->{_finish_res_time}));
+                        1000*($self->{_finish_res_time}-
+                                  $self->{_start_res_time}));
     } else {
         $rest = "-";
     }
@@ -502,7 +485,7 @@ Perinci::Access::Simple::Server::Socket - Implement Riap::Simple server over soc
 
 =head1 VERSION
 
-This document describes version 0.20 of Perinci::Access::Simple::Server::Socket (from Perl distribution Perinci-Access-Simple-Server), released on 2014-12-11.
+This document describes version 0.21 of Perinci::Access::Simple::Server::Socket (from Perl distribution Perinci-Access-Simple-Server), released on 2015-03-05.
 
 =head1 SYNOPSIS
 
@@ -677,7 +660,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
